﻿using System;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Windows.Forms;
using AddinExpress.MSO;
using Outlook = Microsoft.Office.Interop.Outlook;

namespace ReadMail
{
    /// <summary>
    ///   Add-in Express Add-in Module
    /// </summary>
    [GuidAttribute("79F0D0ED-403D-4E84-8F2C-2B24F05C698D"), ProgId("ReadMail.AddinModule")]
    public partial class AddinModule : AddinExpress.MSO.ADXAddinModule
    {
        OutlookItemEventsClass itemEvents = null;
        private bool canConnect = true;
        public AddinModule()
        {
            Application.EnableVisualStyles();
            InitializeComponent();
            // Please add any initialization code to the AddinInitialize event handler
        }

        #region Add-in Express automatic code

        // Required by Add-in Express - do not modify
        // the methods within this region

        public override System.ComponentModel.IContainer GetContainer()
        {
            if (components == null)
                components = new System.ComponentModel.Container();
            return components;
        }

        [ComRegisterFunctionAttribute]
        public static void AddinRegister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXRegister(t);
        }

        [ComUnregisterFunctionAttribute]
        public static void AddinUnregister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXUnregister(t);
        }

        public override void UninstallControls()
        {
            base.UninstallControls();
        }

        #endregion

        public static new AddinModule CurrentInstance
        {
            get
            {
                return AddinExpress.MSO.ADXAddinModule.CurrentInstance as AddinModule;
            }
        }

        public Outlook._Application OutlookApp
        {
            get
            {
                return (HostApplication as Outlook._Application);
            }
        }

        private void AddinModule_AddinStartupComplete(object sender, EventArgs e)
        {
            itemEvents = new OutlookItemEventsClass(this);
        }

        private void adxOutlookEvents_InspectorActivate(object sender,
            object inspector, string folderName)
        {
            if (canConnect)
            {
                Outlook._Inspector olInsp = (Outlook._Inspector)inspector;
                object item = olInsp.CurrentItem;

                if (item is Outlook._MailItem)
                {
                    itemEvents.ConnectTo(item, true);
                }
                else
                    Marshal.ReleaseComObject(item);
            }
        }

        private void ConnectToSelectedItem(object explorer)
        {
            Outlook.Selection sel = null;
            try
            {
                Outlook._Explorer expl = explorer as Outlook._Explorer;
                sel = expl.Selection;
                if (sel.Count == 1)
                {
                    object item = sel[1];
                    if (item is Outlook._MailItem)
                        itemEvents.ConnectTo(item, true);
                    else
                        Marshal.ReleaseComObject(item);
                }
            }
            finally
            {
                if (sel != null)
                    Marshal.ReleaseComObject(sel);
            }
        }

        private void adxOutlookEvents_ExplorerClose(object sender, object explorer)
        {
            int count = 0;
            Outlook._Explorers expls = null;

            try
            {
                expls = OutlookApp.Explorers;
                count = expls.Count;
            }
            catch { }
            finally
            {
                if (expls != null)
                    Marshal.ReleaseComObject(expls);
            }

            if (count == 0)
                canConnect = false;
        }

        private void adxOutlookEvents_ExplorerActivate(object sender, object explorer)
        {
            ConnectToSelectedItem(explorer);
        }

        private void adxOutlookEvents_InspectorClose(object sender, object inspector,
            string folderName)
        {
            if (itemEvents != null)
                itemEvents.RemoveConnection();
        }

        private void AddinModule_AddinBeginShutdown(object sender, EventArgs e)
        {
            if (itemEvents != null)
                itemEvents.RemoveConnection();
        }

        private void adxOutlookEvents_ExplorerSelectionChange(object sender, object explorer)
        {
            ConnectToSelectedItem(explorer);
        }
    }
}

