﻿using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using AddinExpress.MSO;
using Microsoft.Office.Interop.Outlook;
using Outlook = Microsoft.Office.Interop.Outlook;

namespace ReadMail
{
    /// <summary>
    /// Add-in Express Outlook Item Events Class
    /// </summary>
    public class OutlookItemEventsClass : AddinExpress.MSO.ADXOutlookItemEvents
    {
        public OutlookItemEventsClass(AddinExpress.MSO.ADXAddinModule module)
            : base(module)
        {
        }

        public override void ProcessOpen(AddinExpress.MSO.ADXCancelEventArgs e)
        {
            if (this.ItemObj is MailItem)
            {
                Outlook.MailItem mail = (Outlook.MailItem)this.ItemObj;
                ProcessReadCount(mail);
            }
        }

        public override void ProcessPropertyChange(string name)
        {
            if (this.ItemObj is MailItem)
            {
                Outlook.MailItem mail = (Outlook.MailItem)this.ItemObj;
                if (name.ToLower() == "unread")
                {
                    ProcessReadCount(mail);
                }
            }
        }

        public override void ProcessRead()
        {
            if (this.ItemObj is MailItem)
            {
                Outlook.MailItem mail = (Outlook.MailItem)this.ItemObj;
                ProcessReadCount(mail);
            }
        }

        public override void ProcessClose(AddinExpress.MSO.ADXCancelEventArgs e)
        {
            if (this.ItemObj is MailItem)
            {
                Outlook.MailItem mail = (Outlook.MailItem)this.ItemObj;
                if (mail.UnRead)
                {
                    Console.Write("E-mail with subject '" + mail.Subject + "' has NOT been read.");
                }
                else
                {
                    Console.Write("E-mail with subject '" + mail.Subject + "' has been read.");
                }
            }
        }
        
        private void ProcessReadCount(MailItem mail)
        {
            if (this.ItemObj is MailItem)
            {
                Outlook.UserProperties userProperties = null;
                Outlook.UserProperty readCountProperty = null;

                try
                {
                    int readCount = 0;
                    userProperties = mail.UserProperties;
                    readCountProperty = userProperties["ReadCount"];
                    if (readCountProperty == null)
                        readCountProperty = userProperties.Add("ReadCount", OlUserPropertyType.olInteger);

                    readCount = Convert.ToInt32(readCountProperty.Value);
                    if (mail.UnRead)
                    {
                        readCountProperty.Value = 0;
                    }
                    else
                    {
                        readCount++;
                        readCountProperty.Value = readCount;
                    }
                    mail.Save();
                }
                finally
                {
                    if (readCountProperty != null)
                        Marshal.ReleaseComObject(readCountProperty);
                    if (userProperties != null)
                        Marshal.ReleaseComObject(userProperties);
                }
            }
        }

        #region Unused


        public override void ProcessAttachmentAdd(object attachment)
        {
            // TODO: Add some code
        }

        public override void ProcessAttachmentRead(object attachment)
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeAttachmentSave(object attachment, ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeCheckNames(ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }



        public override void ProcessCustomAction(object action, object response, ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessCustomPropertyChange(string name)
        {
            // TODO: Add some code
        }

        public override void ProcessForward(object forward, ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessReply(object response, ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessReplyAll(object response, ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessSend(ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessWrite(ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeDelete(object item, ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessAttachmentRemove(object attachment)
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeAttachmentAdd(object attachment, ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeAttachmentPreview(object attachment, ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeAttachmentRead(object attachment, ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeAttachmentWriteToTempFile(object attachment,
            ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessUnload()
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeAutoSave(ADXCancelEventArgs e)
        {
            // TODO: Add some code
        }

        public override void ProcessBeforeRead()
        {
            // TODO: Add some code
        }

        public override void ProcessAfterWrite()
        {
            // TODO: Add some code
        }

        #endregion

    }
}

